@@ -639,8 +639,12 @@ CommentImpl *DocumentImpl::createComment ( DOMStringImpl* data )
     return new CommentImpl( docPtr(), data );
 }
 
-CDATASectionImpl *DocumentImpl::createCDATASection ( DOMStringImpl* data )
+CDATASectionImpl *DocumentImpl::createCDATASection ( DOMStringImpl* data, int &exceptioncode )
 {
+    if (isHTMLDocument()) {
+        exceptioncode = DOMException::NOT_SUPPORTED_ERR;
+        return 0;
+    }
     return new CDATASectionImpl( docPtr(), data );
 }
 
@@ -649,8 +653,12 @@ ProcessingInstructionImpl *DocumentImpl::createProcessingInstruction ( const DOM
     return new ProcessingInstructionImpl( docPtr(),target,data);
 }
 
-EntityReferenceImpl *DocumentImpl::createEntityReference ( const DOMString &name )
+EntityReferenceImpl *DocumentImpl::createEntityReference ( const DOMString &name, int &exceptioncode )
 {
+    if (isHTMLDocument()) {
+        exceptioncode = DOMException::NOT_SUPPORTED_ERR;
+        return 0;
+    }
     return new EntityReferenceImpl(docPtr(), name.implementation());
 }
 
@@ -702,11 +710,11 @@ NodeImpl *DocumentImpl::importNode(NodeImpl *importedNode, bool deep, int &excep
     }
     else if(importedNode->nodeType() == Node::CDATA_SECTION_NODE)
     {
-	result = createCDATASection(static_cast<CDATASectionImpl*>(importedNode)->string());
+	result = createCDATASection(static_cast<CDATASectionImpl*>(importedNode)->string(), exceptioncode);
 	deep = false;
     }
     else if(importedNode->nodeType() == Node::ENTITY_REFERENCE_NODE)
-	result = createEntityReference(importedNode->nodeName());
+	result = createEntityReference(importedNode->nodeName(), exceptioncode);
     else if(importedNode->nodeType() == Node::PROCESSING_INSTRUCTION_NODE)
     {
 	result = createProcessingInstruction(importedNode->nodeName(), importedNode->nodeValue().implementation());
@@ -1995,7 +2003,7 @@ void DocumentImpl::processHttpEquiv(const DOMString &equiv, const DOMString &con
             str = parseURL( DOMString(str) ).string();
             QString newURL = document()->completeURL( str );
             if ( ok )
-                v->part()->scheduleRedirection(delay, document()->completeURL( str ),  delay < 2 || newURL == URL().url());
+                v->part()->scheduleRedirection(delay, newURL,  delay < 2 || newURL == URL().url());
         }
     }
     else if(strcasecmp(equiv, "expires") == 0)
