@@ -189,6 +189,15 @@ namespace Nepomuk {
          */
         QList<Types::Class> types() const;
 
+        /**
+         * \brief Return the number of types that are being watched
+         *
+         * This method is slightly faster than called types().count()
+         *
+         * \sa types addType setType
+         */
+        int typeCount() const;
+
         /**
          * \brief The resources that have been configured via addResource() and setResources().
          *
@@ -197,6 +206,15 @@ namespace Nepomuk {
          */
         QList<Nepomuk::Resource> resources() const;
 
+        /**
+         * \brief Return the number of resources that are being watched
+         *
+         * This method is slightly faster than called resources().count()
+         *
+         * \sa resources addResource setResources
+         */
+        int resourceCount() const;
+
         /**
          * \brief The properties that have been configured via addProperty() and setProperties().
          *
@@ -205,6 +223,15 @@ namespace Nepomuk {
          */
         QList<Types::Property> properties() const;
 
+        /**
+         * \brief Return the number of properties that are being watched
+         *
+         * This method is slightly faster than called properties().count()
+         *
+         * \sa properties addProperty setProperty
+         */
+        int propertyCount() const;
+
         /**
          * \brief Start the signalling of changes.
          *
